@@ -19,6 +19,12 @@ using namespace KItinerary;
 
 #define s(x) QStringLiteral(x)
 
+#ifdef Q_OS_WINDOWS
+#define LINEBREAK "\r\n"
+#else
+#define LINEBREAK "\n"
+#endif
+
 class ExtractorDocumentNodeTest : public QObject
 {
     Q_OBJECT
@@ -54,9 +60,6 @@ private Q_SLOTS:
 
         QFile f(QStringLiteral(SOURCE_DIR "/misc/test.pdf"));
         QVERIFY(f.open(QFile::ReadOnly));
-#ifndef HAVE_POPPLER
-        QSKIP("No Poppler support");
-#endif
         auto root = engine.documentNodeFactory()->createNode(f.readAll());
         QVERIFY(!root.isNull());
         QCOMPARE(root.mimeType(), QLatin1String("application/pdf"));
@@ -69,7 +72,7 @@ private Q_SLOTS:
         auto c3 = root.childNodes()[2];
         QVERIFY(!c3.isNull());
         QCOMPARE(c3.mimeType(), QLatin1String("text/plain"));
-        QCOMPARE(c3.content<QString>(), QLatin1String("This is the first page.\nIt contains a PDF 417 barcode.\nThis is the second page.\nIt contains an Aztec code.\n"));
+        QCOMPARE(c3.content<QString>(), QLatin1String("This is the first page." LINEBREAK "It contains a PDF 417 barcode." LINEBREAK "This is the second page." LINEBREAK "It contains an Aztec code." LINEBREAK));
 
         auto c1 = root.childNodes()[0];
         QVERIFY(!c1.isNull());
@@ -80,7 +83,7 @@ private Q_SLOTS:
         QCOMPARE(c1.location().toInt(), 0);
 
         c1.processor()->expandNode(c1, &engine);
-#ifndef HAVE_ZXING
+#if !HAVE_ZXING
         QSKIP("No ZXing support");
 #endif
         QCOMPARE(c1.childNodes().size(), 1);
@@ -97,9 +100,6 @@ private Q_SLOTS:
 
         QFile f(QStringLiteral(SOURCE_DIR "/misc/test.pdf"));
         QVERIFY(f.open(QFile::ReadOnly));
-#ifndef HAVE_POPPLER
-        QSKIP("No Poppler support");
-#endif
         std::unique_ptr<PdfDocument> pdf(PdfDocument::fromData(f.readAll()));
         auto root = engine.documentNodeFactory()->createNode(QVariant::fromValue(pdf.get()), u"application/pdf");
         QVERIFY(!root.isNull());
@@ -113,7 +113,7 @@ private Q_SLOTS:
         QCOMPARE(c2.location().toInt(), 1);
 
         c2.processor()->expandNode(c2, &engine);
-#ifndef HAVE_ZXING
+#if !HAVE_ZXING
         QSKIP("No ZXing support");
 #endif
         QCOMPARE(c2.childNodes().size(), 1);
