@@ -16,6 +16,7 @@
 #include <QDebug>
 #include <QFile>
 #include <QJsonDocument>
+#include <QProcess>
 #include <QTest>
 
 using namespace KItinerary;
@@ -54,13 +55,11 @@ private Q_SLOTS:
 
         QTest::newRow("text/plain") << s(SOURCE_DIR "/scriptenginedata/plain-text.txt") << s(SOURCE_DIR "/scriptenginedata/plain-text.txt.json");
         QTest::newRow("pkpass") << s(SOURCE_DIR "/pkpassdata/swiss.pkpass") << s(SOURCE_DIR "/scriptenginedata/swiss.pkpass.json");
-#ifdef HAVE_POPPLER
         QTest::newRow("IATA BCBP PDF") << s(SOURCE_DIR "/extractordata/synthetic/iata-bcbp-demo.pdf")
-#ifdef HAVE_ZXING
+#if HAVE_ZXING
             << s(SOURCE_DIR "/scriptenginedata/iata-bcbp-demo.pdf.json");
 #else
             << s(SOURCE_DIR "/scriptenginedata/iata-bcbp-demo.pdf-no-zxing.json");
-#endif
 #endif
         QTest::newRow("ical") << s(SOURCE_DIR "/extractordata/ical/eventreservation.ics") << s(SOURCE_DIR "/scriptenginedata/eventreservation.ics.json");
         QTest::newRow("uic9183") << s(SOURCE_DIR "/uic918-3/valid/Testw_VOW8XG6_A9E8DXL_0.bin") << s(SOURCE_DIR "/scriptenginedata/Testw_VOW8XG6_A9E8DXL_0.bin.json");
@@ -90,7 +89,15 @@ private Q_SLOTS:
         const auto refResult = QJsonDocument::fromJson(ref.readAll()).array();
 
         if (result != refResult) {
-            qDebug().noquote() << QJsonDocument(result).toJson();
+            QFile failFile(refFile + QLatin1String(".fail"));
+            QVERIFY(failFile.open(QFile::WriteOnly));
+            failFile.write(QJsonDocument(result).toJson());
+            failFile.close();
+
+            QProcess proc;
+            proc.setProcessChannelMode(QProcess::ForwardedChannels);
+            proc.start(QStringLiteral("diff"), {QStringLiteral("-u"), refFile, failFile.fileName()});
+            QVERIFY(proc.waitForFinished());
         }
         QCOMPARE(result, refResult);
     }
