@@ -5,6 +5,7 @@
 */
 
 #include "extractorvalidator.h"
+#include "stringutil.h"
 
 #include <QDateTime>
 #include <QVariant>
@@ -30,6 +31,7 @@ class ProgramMembership;
 class RentalCar;
 class RentalCarReservation;
 class TaxiReservation;
+class Ticket;
 class TouristAttractionVisit;
 class TrainReservation;
 class TrainStation;
@@ -59,6 +61,7 @@ public:
     TaxiReservation processTaxiReservation(TaxiReservation res) const;
     Event processEvent(Event event) const;
 
+    Ticket processTicket(Ticket ticket) const;
     ProgramMembership processProgramMembership(ProgramMembership program) const;
 
     template <typename T> T processReservation(T res) const;
@@ -78,7 +81,7 @@ public:
 
 template<typename T> inline T ExtractorPostprocessorPrivate::processPlace(T place)
 {
-    place.setName(place.name().simplified());
+    place.setName(StringUtil::clean(place.name()));
     auto addr = processAddress(place.address(), place.telephone(), place.geo());
     place.setAddress(addr);
     place.setTelephone(processPhoneNumber(place.telephone(), place.address()));
