@@ -5,10 +5,11 @@
 */
 
 #include "jsonlddocument.h"
-#include "jsonldimportfilter.h"
+#include "json/jsonldimportfilter.h"
 #include "logging.h"
 
 #include <KItinerary/Action>
+#include <KItinerary/BoatTrip>
 #include <KItinerary/Brand>
 #include <KItinerary/BusTrip>
 #include <KItinerary/CreativeWork>
@@ -17,6 +18,7 @@
 #include <KItinerary/Organization>
 #include <KItinerary/Person>
 #include <KItinerary/Place>
+#include <KItinerary/ProgramMembership>
 #include <KItinerary/Reservation>
 #include <KItinerary/RentalCar>
 #include <KItinerary/Taxi>
@@ -30,12 +32,91 @@
 #include <QMetaProperty>
 #include <QUrl>
 #include <QTimeZone>
+#include <QSequentialIterable>
 
+#include <cassert>
 #include <cmath>
+#include <cstring>
 
 using namespace KItinerary;
 
+struct TypeInfo {
+    const char *name;
+    const QMetaObject *mo;
+    int metaTypeId;
+};
+
+static void registerBuiltInTypes(std::vector<TypeInfo> &r);
+static std::vector<TypeInfo>& typeResgistry()
+{
+    static std::vector<TypeInfo> s_typeResgistry;
+    if (s_typeResgistry.empty()) {
+        registerBuiltInTypes(s_typeResgistry);
+    }
+    return s_typeResgistry;
+}
+
+template <typename T>
+static void add(std::vector<TypeInfo> &registry)
+{
+    registry.push_back({ T::typeName(), &T::staticMetaObject, qMetaTypeId<T>() });
+}
+
+static void registerBuiltInTypes(std::vector<TypeInfo> &r)
+{
+    add<Action>(r);
+    add<Airline>(r);
+    add<Airport>(r);
+    add<BoatReservation>(r);
+    add<BoatTerminal>(r);
+    add<BoatTrip>(r);
+    add<Brand>(r);
+    add<BusReservation>(r);
+    add<BusStation>(r);
+    add<BusTrip>(r);
+    add<CancelAction>(r);
+    add<CheckInAction>(r);
+    add<CreativeWork>(r);
+    add<DigitalDocument>(r);
+    add<DownloadAction>(r);
+    add<EmailMessage>(r);
+    add<Event>(r);
+    add<EventReservation>(r);
+    add<Flight>(r);
+    add<FlightReservation>(r);
+    add<FoodEstablishment>(r);
+    add<FoodEstablishmentReservation>(r);
+    add<GeoCoordinates>(r);
+    add<LocalBusiness>(r);
+    add<LodgingBusiness>(r);
+    add<LodgingReservation>(r);
+    add<Organization>(r);
+    add<Person>(r);
+    add<Place>(r);
+    add<PostalAddress>(r);
+    add<ProgramMembership>(r);
+    add<RentalCar>(r);
+    add<RentalCarReservation>(r);
+    add<ReserveAction>(r);
+    add<Seat>(r);
+    add<Taxi>(r);
+    add<TaxiReservation>(r);
+    add<Ticket>(r);
+    add<TouristAttraction>(r);
+    add<TouristAttractionVisit>(r);
+    add<TrainReservation>(r);
+    add<TrainStation>(r);
+    add<TrainTrip>(r);
+    add<UpdateAction>(r);
+    add<ViewAction>(r);
+
+    assert(std::is_sorted(r.begin(), r.end(), [](const auto &lhs, const auto &rhs) {
+        return std::strcmp(lhs.name, rhs.name) < 0;
+    }));
+}
+
 static QVariant createInstance(const QJsonObject &obj);
+static QVariant createInstance(const QJsonObject &obj, const QMetaProperty &prop);
 
 // Eurowings workarounds...
 static const char* const fallbackDateTimePattern[] = {
@@ -107,6 +188,9 @@ static QVariant propertyValue(const QMetaProperty &prop, const QJsonValue &v)
         } else {
             auto str = v.toString();
             dt = QDateTime::fromString(str, Qt::ISODate);
+            if (dt.isNull()) {
+                dt = QDateTime::fromString(QString(str).simplified().remove(QLatin1Char(' ')), Qt::ISODate);
+            }
             for (unsigned int i = 0; i < fallbackDateTimePatternCount && dt.isNull(); ++i) {
                 dt = QDateTime::fromString(str, QString::fromLatin1(fallbackDateTimePattern[i]));
             }
@@ -162,7 +246,7 @@ static QVariant propertyValue(const QMetaProperty &prop, const QJsonValue &v)
     if (prop.userType() == qMetaTypeId<QVariant>() && isEmptyJsonLdObject(obj)) {
         return {};
     }
-    return createInstance(obj);
+    return createInstance(obj, prop);
 }
 
 static void createInstance(const QMetaObject *mo, void *v, const QJsonObject &obj)
@@ -182,62 +266,21 @@ static void createInstance(const QMetaObject *mo, void *v, const QJsonObject &ob
     }
 }
 
-template<typename T>
-static QVariant createInstance(const QJsonObject &obj)
+static QVariant createInstance(const QJsonObject& obj, const QString &type)
 {
-    T t;
-    createInstance(&T::staticMetaObject, &t, obj);
-    return QVariant::fromValue(t);
-}
-
-#define MAKE_FACTORY(Class) \
-    if (type == QLatin1String(#Class)) \
-        return createInstance<Class>(obj)
-
-static QVariant createInstance(const QJsonObject &obj)
-{
-    const auto type = obj.value(QLatin1String("@type")).toString();
-    MAKE_FACTORY(Action);
-    MAKE_FACTORY(Airline);
-    MAKE_FACTORY(Airport);
-    MAKE_FACTORY(Brand);
-    MAKE_FACTORY(BusReservation);
-    MAKE_FACTORY(BusStation);
-    MAKE_FACTORY(BusTrip);
-    MAKE_FACTORY(CancelAction);
-    MAKE_FACTORY(CheckInAction);
-    MAKE_FACTORY(CreativeWork);
-    MAKE_FACTORY(DigitalDocument);
-    MAKE_FACTORY(DownloadAction);
-    MAKE_FACTORY(EmailMessage);
-    MAKE_FACTORY(Event);
-    MAKE_FACTORY(EventReservation);
-    MAKE_FACTORY(Flight);
-    MAKE_FACTORY(FlightReservation);
-    MAKE_FACTORY(FoodEstablishment);
-    MAKE_FACTORY(FoodEstablishmentReservation);
-    MAKE_FACTORY(LocalBusiness);
-    MAKE_FACTORY(RentalCarReservation);
-    MAKE_FACTORY(RentalCar);
-    MAKE_FACTORY(ReserveAction);
-    MAKE_FACTORY(GeoCoordinates);
-    MAKE_FACTORY(LodgingBusiness);
-    MAKE_FACTORY(LodgingReservation);
-    MAKE_FACTORY(Organization);
-    MAKE_FACTORY(Person);
-    MAKE_FACTORY(Place);
-    MAKE_FACTORY(PostalAddress);
-    MAKE_FACTORY(Seat);
-    MAKE_FACTORY(TaxiReservation);
-    MAKE_FACTORY(Taxi);
-    MAKE_FACTORY(Ticket);
-    MAKE_FACTORY(TouristAttraction);
-    MAKE_FACTORY(TouristAttractionVisit);
-    MAKE_FACTORY(TrainReservation);
-    MAKE_FACTORY(TrainStation);
-    MAKE_FACTORY(TrainTrip);
-    MAKE_FACTORY(UpdateAction);
-    MAKE_FACTORY(ViewAction);
+    const auto& registry = typeResgistry();
+    const auto it = std::lower_bound(registry.begin(), registry.end(), type, [](const auto &lhs, const auto &rhs) {
+        return QLatin1String(lhs.name) < rhs;
+    });
+    if (it != registry.end() && QLatin1String((*it).name) == type) {
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+        QVariant value((*it).metaTypeId, nullptr);
+#else
+        QVariant value(QMetaType((*it).metaTypeId), nullptr);
+#endif
+        createInstance((*it).mo, value.data(), obj);
+        return value;
+    }
 
     if (type == QLatin1String("QDateTime")) {
         auto dt = QDateTime::fromString(obj.value(QLatin1String("@value")).toString(), Qt::ISODate);
@@ -247,9 +290,45 @@ static QVariant createInstance(const QJsonObject &obj)
 
     return {};
 }
-#undef MAKE_FACTORY
 
-static QVector<QVariant> fromJson(const QJsonObject &obj) // TODO this should replace JsonLdDocument::fromJson once we can change the API
+static QVariant createInstance(const QJsonObject &obj)
+{
+    return createInstance(obj, obj.value(QLatin1String("@type")).toString());
+}
+
+static QVariant createInstance(const QJsonObject &obj, const QMetaProperty &prop)
+{
+    const auto type = obj.value(QLatin1String("@type")).toString();
+    if (!type.isEmpty()) {
+        return createInstance(obj, type);
+    }
+
+    // if @type is (wrongly) not specified, try to recover from our own knowledge of a property type
+    const auto mo = QMetaType::metaObjectForType(prop.userType());
+    if (mo) {
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+        QVariant value(prop.userType(), nullptr);
+#else
+        QVariant value(prop.metaType(), nullptr);
+#endif
+        createInstance(mo, value.data(), obj);
+        return value;
+    }
+
+    return {};
+}
+
+QVector<QVariant> JsonLdDocument::fromJson(const QJsonArray &array)
+{
+    QVector<QVariant> l;
+    l.reserve(array.size());
+    for (const auto &obj : array) {
+        l.append(JsonLdDocument::fromJson(obj.toObject()));
+    }
+    return l;
+}
+
+QVector<QVariant> JsonLdDocument::fromJson(const QJsonObject &obj)
 {
     const auto normalized = JsonLdImportFilter::filterObject(obj);
     QVector<QVariant> result;
@@ -263,21 +342,6 @@ static QVector<QVariant> fromJson(const QJsonObject &obj) // TODO this should re
     return result;
 }
 
-QVector<QVariant> JsonLdDocument::fromJson(const QJsonArray &array)
-{
-    QVector<QVariant> l;
-    l.reserve(array.size());
-    for (const auto &obj : array) {
-        l.append(::fromJson(obj.toObject()));
-    }
-    return l;
-}
-
-QVariant JsonLdDocument::fromJson(const QJsonObject& obj)
-{
-    return fromJsonSingular(obj); // ### temporary, see above
-}
-
 QVariant JsonLdDocument::fromJsonSingular(const QJsonObject &obj)
 {
     const auto normalized = JsonLdImportFilter::filterObject(obj);
@@ -287,7 +351,7 @@ QVariant JsonLdDocument::fromJsonSingular(const QJsonObject &obj)
     return createInstance(normalized.at(0).toObject());
 }
 
-static bool valueIsNull(const QVariant &v)
+bool JsonLd::valueIsNull(const QVariant &v)
 {
     if (v.type() == QVariant::Url) {
         return !v.toUrl().isValid();
@@ -295,6 +359,19 @@ static bool valueIsNull(const QVariant &v)
     if (v.type() == qMetaTypeId<float>()) {
         return std::isnan(v.toFloat());
     }
+    if (v.type() == QVariant::List) {
+        return v.toList().isEmpty();
+    }
+    // starting with Qt6, QVariant::isNull is "shallow" and would miss the following as well
+    if (v.type() == QVariant::String) {
+        return v.toString().isNull();
+    }
+    if (v.type() == QVariant::DateTime) {
+        return v.toDateTime().isNull();
+    }
+    if (v.type() == QVariant::Date) {
+        return v.toDate().isNull();
+    }
     return v.isNull();
 }
 
@@ -311,7 +388,7 @@ static QString typeName(const QMetaObject *mo, const QVariant &v)
     return QString::fromUtf8(mo->className());
 }
 
-static QJsonValue toJsonValue(const QVariant &v)
+QJsonValue JsonLdDocument::toJsonValue(const QVariant &v)
 {
     const auto mo = QMetaType(v.userType()).metaObject();
     if (!mo) {
@@ -390,7 +467,7 @@ static QJsonValue toJsonValue(const QVariant &v)
         }
 
         const auto value = prop.readOnGadget(v.constData());
-        if (!valueIsNull(value)) {
+        if (!JsonLd::valueIsNull(value)) {
             const auto jsVal = toJsonValue(value);
             if (jsVal.type() != QJsonValue::Null) {
                 obj.insert(QString::fromUtf8(prop.name()), jsVal);
@@ -408,7 +485,7 @@ QJsonArray JsonLdDocument::toJson(const QVector<QVariant> &data)
 {
     QJsonArray a;
     for (const auto &d : data) {
-        const auto value = ::toJsonValue(d);
+        const auto value = toJsonValue(d);
         if (!value.isObject()) {
             continue;
         }
@@ -421,7 +498,7 @@ QJsonArray JsonLdDocument::toJson(const QVector<QVariant> &data)
 
 QJsonObject JsonLdDocument::toJson(const QVariant& data)
 {
-    const auto value = ::toJsonValue(data);
+    const auto value = toJsonValue(data);
     if (!value.isObject()) {
         return {};
     }
@@ -519,19 +596,36 @@ QVariant JsonLdDocument::apply(const QVariant& lhs, const QVariant& rhs)
             }
             auto valueData = mt.create();
             *reinterpret_cast<int*>(valueData) = numValue;
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
             QVariant value(prop.userType(), valueData);
+#else
+            QVariant value(prop.metaType(), valueData);
+#endif
             prop.writeOnGadget(res.data(), value);
             continue;
         }
 
         auto pv = prop.readOnGadget(rhs.constData());
-        if (QMetaType(pv.userType()).metaObject()) {
+        if ((QMetaType::typeFlags(pv.userType()) & QMetaType::IsGadget) && QMetaType(pv.userType()).metaObject()) {
             pv = apply(prop.readOnGadget(lhs.constData()), pv);
         }
-        if (!pv.isNull()) {
+        if (!JsonLd::valueIsNull(pv)) {
             prop.writeOnGadget(res.data(), pv);
         }
     }
 
     return res;
 }
+
+void JsonLdDocument::registerType(const char *typeName, const QMetaObject *mo, int metaTypeId)
+{
+    auto &registry = typeResgistry();
+    const auto it = std::lower_bound(registry.begin(), registry.end(), typeName, [](const auto &lhs, const auto *rhs) {
+        return std::strcmp(lhs.name, rhs) < 0;
+    });
+    if (it != registry.end() && std::strcmp((*it).name, typeName) == 0) {
+        qCWarning(Log) << "Type already registered:" << typeName;
+    } else {
+        registry.insert(it, { typeName, mo, metaTypeId });
+    }
+}
