@@ -29,6 +29,7 @@
 #include <QDebug>
 #include <QMetaObject>
 #include <QMetaProperty>
+#include <QTimeZone>
 #include <QVariant>
 
 #include <cmath>
@@ -74,9 +75,28 @@ static bool isSameTouristAttraction(const TouristAttraction &lhs, const TouristA
 static bool isSameEvent(const Event &lhs, const Event &rhs);
 static bool isSameRentalCar(const RentalCar &lhs, const RentalCar &rhs);
 static bool isSameTaxiTrip(const Taxi &lhs, const Taxi &rhs);
+static bool isSameReservation(const Reservation &lhsRes, const Reservation &rhsRes);
 static bool isMinimalCancelationFor(const QVariant &r, const Reservation &cancel);
 static bool isSameTicketToken(const QVariant &lhs, const QVariant &rhs);
 
+bool isSameReservation(const Reservation &lhsRes, const Reservation &rhsRes)
+{
+    // underName either matches or is not set
+    const auto lhsUN = lhsRes.underName().value<Person>();
+    const auto rhsUN = rhsRes.underName().value<Person>();
+    if (!lhsUN.name().isEmpty() && !rhsUN.name().isEmpty() &&  !MergeUtil::isSamePerson(lhsUN, rhsUN)) {
+        return false;
+    }
+
+    const auto lhsTicket = lhsRes.reservedTicket().value<Ticket>();
+    const auto rhsTicket = rhsRes.reservedTicket().value<Ticket>();
+    if (conflictIfPresent(lhsTicket.ticketedSeat().seatNumber(), rhsTicket.ticketedSeat().seatNumber(), Qt::CaseInsensitive)) {
+        return false;
+    }
+
+    return true;
+}
+
 bool MergeUtil::isSame(const QVariant& lhs, const QVariant& rhs)
 {
     if (lhs.isNull() || rhs.isNull()) {
@@ -90,19 +110,12 @@ bool MergeUtil::isSame(const QVariant& lhs, const QVariant& rhs)
     if (JsonLd::canConvert<Reservation>(lhs)) {
         const auto lhsRes = JsonLd::convert<Reservation>(lhs);
         const auto rhsRes = JsonLd::convert<Reservation>(rhs);
-
-        // for all: underName either matches or is not set
-        const auto lhsUN = lhsRes.underName().value<Person>();
-        const auto rhsUN = rhsRes.underName().value<Person>();
-        if (!lhsUN.name().isEmpty() && !rhsUN.name().isEmpty() &&  !isSamePerson(lhsUN, rhsUN)) {
+        if (!isSameReservation(lhsRes, rhsRes)) {
             return false;
         }
 
         const auto lhsTicket = lhsRes.reservedTicket().value<Ticket>();
         const auto rhsTicket = rhsRes.reservedTicket().value<Ticket>();
-        if (conflictIfPresent(lhsTicket.ticketedSeat().seatNumber(), rhsTicket.ticketedSeat().seatNumber(), Qt::CaseInsensitive)) {
-            return false;
-        }
         // flight ticket tokens (IATA BCBP) can differ, so we need to compare the relevant bits in them manually
         // this however happens automatically as they are unpacked to other fields by post-processing
         // so we can simply skip this here for flights
@@ -251,6 +264,27 @@ bool MergeUtil::isSame(const QVariant& lhs, const QVariant& rhs)
         return isSameTouristAttractionVisit(l, r);
     }
 
+    // top-level tickets
+    if (JsonLd::isA<Ticket>(lhs)) {
+        const auto lhsTicket = lhs.value<Ticket>();
+        const auto rhsTicket = rhs.value<Ticket>();
+
+        // underName either matches or is not set
+        const auto lhsUN = lhsTicket.underName();
+        const auto rhsUN = rhsTicket.underName();
+        if (!lhsUN.name().isEmpty() && !rhsUN.name().isEmpty() &&  !isSamePerson(lhsUN, rhsUN)) {
+            return false;
+        }
+
+        if (conflictIfPresent(lhsTicket.ticketNumber(), rhsTicket.ticketNumber())
+         || conflictIfPresent(lhsTicket.name(), rhsTicket.name())
+         || conflictIfPresent(lhsTicket.validFrom(), rhsTicket.validFrom())
+         || !isSameTicketToken(lhsTicket.ticketTokenData(), rhsTicket.ticketTokenData())
+        ) {
+            return false;
+        }
+    }
+
     return true;
 }
 
@@ -328,14 +362,17 @@ static bool isSameTrainTrip(const TrainTrip &lhs, const TrainTrip &rhs)
     // so we have to use the slightly less robust location comparison
     if (!lhs.departureTime().isValid() && !rhs.departureTime().isValid()) {
         qCDebug(CompareLog) << "unbound trip" << lhs.departureStation().name() << rhs.departureStation().name() << lhs.arrivalStation().name() << rhs.arrivalStation().name();
-        return lhs.departureStation().name() == rhs.departureStation().name() && lhs.arrivalStation().name() == rhs.arrivalStation().name();
-    } else if (!equalAndPresent(lhs.departureTime(), rhs.departureTime())) {
+        return LocationUtil::isSameLocation(lhs.departureStation(), rhs.departureStation(), LocationUtil::Exact)
+            && LocationUtil::isSameLocation(lhs.arrivalStation(), rhs.arrivalStation(), LocationUtil::Exact);
+    } else if (!equalAndPresent(lhs.departureTime(), rhs.departureTime()) || conflictIfPresent(lhs.arrivalTime(), rhs.arrivalTime())) {
         return false;
     }
 
+    // if we don't have train numbers, also fall back to the less robust location comparison
     if (lhs.trainNumber().isEmpty() || rhs.trainNumber().isEmpty()) {
         qCDebug(CompareLog) << "missing train number" << lhs.trainNumber() << rhs.trainNumber();
-        return false;
+        return LocationUtil::isSameLocation(lhs.departureStation(), rhs.departureStation(), LocationUtil::Exact)
+            && LocationUtil::isSameLocation(lhs.arrivalStation(), rhs.arrivalStation(), LocationUtil::Exact);
     }
 
     const auto isSameLine = isSameLineName(lhs.trainNumber(), rhs.trainNumber());
@@ -469,6 +506,10 @@ static Airline mergeValue(const Airline &lhs, const Airline &rhs)
 
 static QDateTime mergeValue(const QDateTime &lhs, const QDateTime &rhs)
 {
+    // if both sides have a timezone, prefer non-UTC
+    if (lhs.isValid() && lhs.timeSpec() == Qt::TimeZone && rhs.isValid() && rhs.timeSpec() == Qt::TimeZone) {
+        return rhs.timeZone() == QTimeZone::utc() ? lhs : rhs;
+    }
     // prefer value with timezone
     return lhs.isValid() && lhs.timeSpec() == Qt::TimeZone && rhs.timeSpec() != Qt::TimeZone ? lhs : rhs;
 }
@@ -486,20 +527,12 @@ static Ticket mergeValue(const Ticket &lhs, const Ticket &rhs)
 {
     auto t = JsonLdDocument::apply(lhs, rhs).value<Ticket>();
     // prefer barcode ticket tokens over URLs
-    if (t.ticketTokenType() == Ticket::Url && lhs.ticketTokenType() != Ticket::Url && lhs.ticketTokenType() != Ticket::Unknown) {
+    if (t.ticketTokenType() == Token::Url && lhs.ticketTokenType() != Token::Url && lhs.ticketTokenType() != Token::Unknown) {
         t.setTicketToken(lhs.ticketToken());
     }
     return t;
 }
 
-static bool checkValueIsNull(const QVariant &v)
-{
-    if (v.type() == qMetaTypeId<float>()) {
-        return std::isnan(v.toFloat());
-    }
-    return v.isNull();
-}
-
 QVariant MergeUtil::merge(const QVariant &lhs, const QVariant &rhs)
 {
     if (rhs.isNull()) {
@@ -533,6 +566,7 @@ QVariant MergeUtil::merge(const QVariant &lhs, const QVariant &rhs)
         auto lv = prop.readOnGadget(lhs.constData());
         auto rv = prop.readOnGadget(rhs.constData());
         auto mt = rv.userType();
+        const auto metaType = QMetaType(mt);
 
         if (mt == qMetaTypeId<Airline>()) {
             rv = mergeValue(lv.value<Airline>(), rv.value<Airline>());
@@ -542,11 +576,11 @@ QVariant MergeUtil::merge(const QVariant &lhs, const QVariant &rhs)
             rv = mergeValue(lv.toDateTime(), rv.toDateTime());
         } else if (mt == qMetaTypeId<Ticket>()) {
             rv = mergeValue(lv.value<Ticket>(), rv.value<Ticket>());
-        } else if (QMetaType(mt).metaObject()) {
+        } else if ((metaType.flags() & QMetaType::IsGadget) && metaType.metaObject()) {
             rv = merge(prop.readOnGadget(lhs.constData()), rv);
         }
 
-        if (!checkValueIsNull(rv)) {
+        if (!JsonLd::valueIsNull(rv)) {
             prop.writeOnGadget(res.data(), rv);
         }
     }
@@ -588,3 +622,38 @@ bool isSameTicketToken(const QVariant &lhs, const QVariant &rhs)
     qCWarning(CompareLog) << "unhandled ticket token type" << lhs << rhs;
     return false;
 }
+
+static bool isCompatibleLocationChange(const QVariant &lhs, const QVariant &rhs)
+{
+    const bool lhsTrainOrBus = JsonLd::isA<TrainReservation>(lhs) || JsonLd::isA<BusReservation>(lhs);
+    const bool rhsTrainOrBus = JsonLd::isA<TrainReservation>(rhs) || JsonLd::isA<BusReservation>(rhs);
+    return (lhsTrainOrBus && rhsTrainOrBus) || (JsonLd::isA<FlightReservation>(lhs) && JsonLd::isA<FlightReservation>(rhs));
+}
+
+bool MergeUtil::hasSameDeparture(const QVariant &lhs, const QVariant &rhs)
+{
+    if (!isCompatibleLocationChange(lhs, rhs)) {
+        return false;
+    }
+    const auto lhsRes = JsonLd::convert<Reservation>(lhs);
+    const auto rhsRes = JsonLd::convert<Reservation>(rhs);
+    if (!isSameReservation(lhsRes, rhsRes) || SortUtil::startDateTime(lhs) != SortUtil::startDateTime(rhs)) {
+        return false;
+    }
+
+    return LocationUtil::isSameLocation(LocationUtil::departureLocation(lhs), LocationUtil::departureLocation(rhs), LocationUtil::Exact);
+}
+
+bool MergeUtil::hasSameArrival(const QVariant &lhs, const QVariant &rhs)
+{
+    if (!isCompatibleLocationChange(lhs, rhs)) {
+        return false;
+    }
+    const auto lhsRes = JsonLd::convert<Reservation>(lhs);
+    const auto rhsRes = JsonLd::convert<Reservation>(rhs);
+    if (!isSameReservation(lhsRes, rhsRes) || SortUtil::endDateTime(lhs) != SortUtil::endDateTime(rhs)) {
+        return false;
+    }
+
+    return LocationUtil::isSameLocation(LocationUtil::arrivalLocation(lhs), LocationUtil::arrivalLocation(rhs), LocationUtil::Exact);
+}
