@@ -16,6 +16,7 @@ const tariffs = {
     'EF99': 'Carte Enfant+',
     'IR00': 'Interrail',
     'IR01': 'Interrail',
+    'JE00': 'Carte Jeune',
     'SE11': 'Carte Advantage Senior',
     'SR50': 'Carte Senior'
 };
@@ -80,7 +81,7 @@ function parseInouiPdfText(page)
         return reservations;
     var pos = date.index + date[0].length;
     while (true) {
-        var dep = text.substr(pos).match(/(\d+[h:]\d+) +(.*)\n/);
+        var dep = text.substr(pos).match(/(\d{2}[h:]\d{2}) +(.*)\n/);
         if (!dep)
             break;
         pos += dep.index + dep[0].length;
@@ -89,7 +90,7 @@ function parseInouiPdfText(page)
         res.reservationFor.departureTime = JsonLd.toDateTime(date[1] + dep[1], ["d MMMM yyyyhh'h'mm", "dd MMMM yyyyhh:mm"], ["fr", "en"]);
         res.reservationFor.departureStation.name = dep[2];
 
-        var arr = text.substr(pos).match(/(\d+[h:]\d+) +(.*)\n/);
+        var arr = text.substr(pos).match(/(\d{2}[h:]\d{2}) +(.*)\n/);
         if (!arr)
             break;
         var endPos = arr.index + arr[0].length;
@@ -507,6 +508,8 @@ function parseSncfCartePdf(pdf, node, barcode) {
     carte.member = JsonLd.newObject("Person");
     carte.member.familyName = text.match(/Nom\s*:\s*(.*)/)[1];
     carte.member.givenName = text.match(/Prénom\s*:\s*(.*)/)[1];
-    // TODO validity period, once the data model supports that
+    const validity = text.match(/Du\s+(\d{2}\/\d{2}\/\d{4})\s+au\s+(\d{2}\/\d{2}\/\d{4})/);
+    carte.validFrom = JsonLd.toDateTime(validity[1], 'dd/MM/yyyy', 'fr');
+    carte.validTo = JsonLd.toDateTime(validity[2] + ' 23:59:59', 'dd/MM/yyyy hh:mm:ss', 'fr');
     return carte;
 }
